@@ -167,12 +167,14 @@ class CloudConfig(object):
         # What's even more amazing is that they did it AGAIN with cinder v3
         # And then I learned that mistral copied it.
         if service_type == 'volume':
-            if self.get_api_version(service_type).startswith('2'):
+            vol_type = self.get_api_version(service_type)
+            if vol_type and vol_type.startswith('2'):
                 service_type = 'volumev2'
-            elif self.get_api_version(service_type).startswith('3'):
+            elif vol_type and vol_type.startswith('3'):
                 service_type = 'volumev3'
         elif service_type == 'workflow':
-            if self.get_api_version(service_type).startswith('2'):
+            wk_type = self.get_api_version(service_type)
+            if wk_type and wk_type.startswith('2'):
                 service_type = 'workflowv2'
         return self.config.get(key, service_type)
 
@@ -215,17 +217,47 @@ class CloudConfig(object):
             requestsexceptions.squelch_warnings(insecure_requests=not verify)
             self._keystone_session = self._session_constructor(
                 auth=self._auth,
-                app_name=self._app_name,
-                app_version=self._app_version,
                 verify=verify,
                 cert=cert,
                 timeout=self.config['api_timeout'])
             if hasattr(self._keystone_session, 'additional_user_agent'):
                 self._keystone_session.additional_user_agent.append(
                     ('os-client-config', os_client_config.__version__))
+            # Using old keystoneauth with new os-client-config fails if
+            # we pass in app_name and app_version. Those are not essential,
+            # nor a reason to bump our minimum, so just test for the session
+            # having the attribute post creation and set them then.
+            if hasattr(self._keystone_session, 'app_name'):
+                self._keystone_session.app_name = self._app_name
+            if hasattr(self._keystone_session, 'app_version'):
+                self._keystone_session.app_version = self._app_version
         return self._keystone_session
 
-    def get_session_client(self, service_key):
+    def get_service_catalog(self):
+        """Helper method to grab the service catalog."""
+        return self._auth.get_access(self.get_session()).service_catalog
+
+    def _get_version_args(self, service_key, version):
+        """Translate OCC version args to those needed by ksa adapter.
+
+        If no version is requested explicitly and we have a configured version,
+        set the version parameter and let ksa deal with expanding that to
+        min=ver.0, max=ver.latest.
+
+        If version is set, pass it through.
+
+        If version is not set and we don't have a configured version, default
+        to latest.
+        """
+        if version == 'latest':
+            return None, None, 'latest'
+        if not version:
+            version = self.get_api_version(service_key)
+        if not version:
+            return None, None, 'latest'
+        return version, None, None
+
+    def get_session_client(self, service_key, version=None):
         """Return a prepped requests adapter for a given service.
 
         This is useful for making direct requests calls against a
@@ -239,12 +271,17 @@ class CloudConfig(object):
 
         and it will work like you think.
         """
+        (version, min_version, max_version) = self._get_version_args(
+            service_key, version)
 
         return adapter.Adapter(
             session=self.get_session(),
             service_type=self.get_service_type(service_key),
             service_name=self.get_service_name(service_key),
             interface=self.get_interface(service_key),
+            version=version,
+            min_version=min_version,
+            max_version=max_version,
             region_name=self.region)
 
     def _get_highest_endpoint(self, service_types, kwargs):
@@ -360,12 +397,6 @@ class CloudConfig(object):
             service_key, min_version=min_version, max_version=max_version)
         endpoint_override = self.get_endpoint(service_key)
 
-        if not interface_key:
-            if service_key in ('image', 'key-manager', 'identity'):
-                interface_key = 'interface'
-            else:
-                interface_key = 'endpoint_type'
-
         if service_key == 'object-store':
             constructor_kwargs = dict(
                 session=self.get_session(),
@@ -398,10 +429,6 @@ class CloudConfig(object):
             if not endpoint_override:
                 constructor_kwargs['endpoint_override'] = endpoint
         constructor_kwargs.update(kwargs)
-        if service_key == 'object-store':
-            constructor_kwargs['os_options'][interface_key] = interface
-        else:
-            constructor_kwargs[interface_key] = interface
         if pass_version_arg and service_key != 'object-store':
             if not version:
                 version = self.get_api_version(service_key)
@@ -445,6 +472,19 @@ class CloudConfig(object):
             constructor_kwargs['username'] = None
             constructor_kwargs['password'] = None
 
+        if not interface_key:
+            if service_key in ('image', 'key-manager'):
+                interface_key = 'interface'
+            elif (service_key == 'identity'
+                  and version and version.startswith('3')):
+                interface_key = 'interface'
+            else:
+                interface_key = 'endpoint_type'
+        if service_key == 'object-store':
+            constructor_kwargs['os_options'][interface_key] = interface
+        else:
+            constructor_kwargs[interface_key] = interface
+
         return client_class(**constructor_kwargs)
 
     def get_cache_expiration_time(self):
@@ -492,7 +532,7 @@ class CloudConfig(object):
         """
         if self.config['floating_ip_source'] == "None":
             return False
-        return self.config['requires_floating_ip']
+        return self.config.get('requires_floating_ip')
 
     def get_external_networks(self):
         """Get list of network names for external networks."""
@@ -543,3 +583,10 @@ class CloudConfig(object):
             if net['nat_destination']:
                 return net['name']
         return None
+
+    def get_nat_source(self):
+        """Get network used for NAT source."""
+        for net in self.config['networks']:
+            if net.get('nat_source'):
+                return net['name']
+        return None
